<?php

declare(strict_types=1);

namespace Drupal\iconify_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Iconify\IconsJSON\Finder;

/**
 * Plugin implementation of the 'icon_picker' widget.
 *
 * @FieldWidget(
 *   id = "iconify_field_icon_picker",
 *   label = @Translation("Icon Picker"),
 *   field_types = {
 *     "iconify_field_icon",
 *   }
 * )
 */
class IconPicker extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'default_collection' => NULL,
      'collections' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['default_collection'] = [
      '#type' => 'select',
      '#title' => $this->t('Default collection'),
      '#description' => $this->t('The default collection to show.'),
      '#default_value' => $this->getSetting('default_collection'),
      '#options' => [
        NULL => $this->t('First available'),
        ...$this->getIconifyCollections(),
      ],
    ];

    $element['collections'] = [
      '#type' => 'select',
      '#title' => $this->t('Allowed collections'),
      '#description' => $this->t('Select the icon collections that can be used. If none are selected, all collections will be available.'),
      '#default_value' => $this->getSetting('collections'),
      '#options' => $this->getIconifyCollections(),
      '#multiple' => TRUE,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $collections = $this->getSetting('collections');

    $summary = [
      'default_collection' => $this->t('Default collection: @collection', [
        '@collection' => $this->getSetting('default_collection') ?: $this->t('First available'),
      ]),
      'collections' => $this->t('Allowed collections: @collections', [
        '@collections' => !empty($collections) ? implode(', ', $collections) : $this->t('All'),
      ]),
    ];

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $element['#attached'] = [
      'library' => [
        'iconify_field/icon-preview',
        'iconify_field/icon-picker',
      ],
    ];

    $element['value'] = $element + [
      '#type' => 'iconify_field',
      '#default_collection' => $this->getSetting('default_collection'),
      '#collections' => $this->getSetting('collections'),
      '#default_value' => $items[$delta]->value ?? '',
    ];

    return $element;
  }

  /**
   * Get a list of available Iconify collections.
   *
   * @return array
   *   An array of available collections.
   */
  protected function getIconifyCollections(): array {
    $collections = [];

    $collections_data = Finder::collections();

    foreach ($collections_data as $collection_id => $collection) {
      $collection_name = sprintf('%s - %s', $collection['name'], $collection['license']['title']);
      $collections[$collection['category'] ?? 'Misc'][$collection_id] = $collection_name;
    }

    return $collections;
  }

}
